@@ -1,5 +1,4 @@
-from collections import Sequence
-from functools import total_ordering
+from collections import Sequence, Iterable
 import fnmatch
 import linecache
 import os.path
@@ -10,6 +9,38 @@ from _tracemalloc import *
 from _tracemalloc import _get_object_traceback, _get_traces, __version__
 
 
+try:
+    from functools import total_ordering
+except ImportError:
+    # Python 2.6
+    def total_ordering(cls):
+        # Function backported from Python 2.7
+        convert = {
+            '__lt__': [('__gt__', lambda self, other: _not_op_and_not_eq(self.__lt__, self, other)),
+                       ('__le__', lambda self, other: _op_or_eq(self.__lt__, self, other)),
+                       ('__ge__', lambda self, other: _not_op(self.__lt__, other))],
+            '__le__': [('__ge__', lambda self, other: _not_op_or_eq(self.__le__, self, other)),
+                       ('__lt__', lambda self, other: _op_and_not_eq(self.__le__, self, other)),
+                       ('__gt__', lambda self, other: _not_op(self.__le__, other))],
+            '__gt__': [('__lt__', lambda self, other: _not_op_and_not_eq(self.__gt__, self, other)),
+                       ('__ge__', lambda self, other: _op_or_eq(self.__gt__, self, other)),
+                       ('__le__', lambda self, other: _not_op(self.__gt__, other))],
+            '__ge__': [('__le__', lambda self, other: _not_op_or_eq(self.__ge__, self, other)),
+                       ('__gt__', lambda self, other: _op_and_not_eq(self.__ge__, self, other)),
+                       ('__lt__', lambda self, other: _not_op(self.__ge__, other))]
+        }
+        roots = [op for op in convert if getattr(cls, op, None) is not getattr(object, op, None)]
+        if not roots:
+            raise ValueError('must define at least one ordering operation: < > <= >=')
+        root = max(roots)
+        for opname, opfunc in convert[root]:
+            if opname not in roots:
+                opfunc.__name__ = opname
+                opfunc.__doc__ = getattr(int, opname).__doc__
+                setattr(cls, opname, opfunc)
+        return cls
+
+
 def _format_size(size, sign):
     for unit in ('B', 'KiB', 'MiB', 'GiB', 'TiB'):
         if abs(size) < 100 and unit != 'B':
@@ -119,12 +150,12 @@ def _compare_grouped_stats(old_group, new_group):
         previous = old_group.pop(traceback, None)
         if previous is not None:
             stat = StatisticDiff(traceback,
-                                  stat.size, stat.size - previous.size,
-                                  stat.count, stat.count - previous.count)
+                                 stat.size, stat.size - previous.size,
+                                 stat.count, stat.count - previous.count)
         else:
             stat = StatisticDiff(traceback,
-                                  stat.size, stat.size,
-                                  stat.count, stat.count)
+                                 stat.size, stat.size,
+                                 stat.count, stat.count)
         statistics.append(stat)
 
     for traceback, stat in old_group.items():
@@ -141,6 +172,7 @@ class Frame(object):
     __slots__ = ("_frame",)
 
     def __init__(self, frame):
+        # frame is a tuple: (filename: str, lineno: int)
         self._frame = frame
 
     @property
@@ -177,6 +209,8 @@ class Traceback(Sequence):
 
     def __init__(self, frames):
         Sequence.__init__(self)
+        # frames is a tuple of frame tuples: see Frame constructor for the
+        # format of a frame tuple
         self._frames = frames
 
     def __len__(self):
@@ -241,6 +275,8 @@ class Trace(object):
     __slots__ = ("_trace",)
 
     def __init__(self, trace):
+        # trace is a tuple: (size, traceback), see Traceback constructor
+        # for the format of the traceback tuple
         self._trace = trace
 
     @property
@@ -268,6 +304,7 @@ class Trace(object):
 class _Traces(Sequence):
     def __init__(self, traces):
         Sequence.__init__(self)
+        # traces is a tuple of trace tuples: see Trace constructor
         self._traces = traces
 
     def __len__(self):
@@ -338,6 +375,8 @@ class Snapshot(object):
     """
 
     def __init__(self, traces, traceback_limit):
+        # traces is a tuple of trace tuples: see _Traces constructor for
+        # the exact format
         self.traces = _Traces(traces)
         self.traceback_limit = traceback_limit
 
@@ -374,6 +413,9 @@ class Snapshot(object):
         is a list of Filter instances.  If filters is an empty list, return a
         new Snapshot instance with a copy of the traces.
         """
+        if not isinstance(filters, Iterable):
+            raise TypeError("filters must be a list of filters, not %s"
+                            % type(filters).__name__)
         if filters:
             include_filters = []
             exclude_filters = []
